@@ -35,6 +35,7 @@
 #include "midgard_core_xml.h"
 #include "midgard_metadata.h"
 #include "midgard_reflector_object.h"
+#include "midgard_base_abstract.h"
 
 /* TODO tune header files , no need to include string.h while we need to include midgard.h in almost every case */
 
@@ -111,6 +112,7 @@ static const gchar *mgd_complextype[] = {
 	"include", 
 	"copy", 
 	"extends", 
+	TYPE_RW_ABSTRACT,
 	TYPE_RW_USERVALUES,
 	NULL 
 };
@@ -122,6 +124,7 @@ static const gchar *mgd_attribute[] = {
 	TYPE_RW_EXTENDS,
 	TYPE_RW_COPY,
 	TYPE_RW_METADATA,
+	TYPE_RW_ABSTRACT,
 	"parent", 
 	"parentfield", 
 	"link", 
@@ -174,6 +177,15 @@ static const gchar *rcolumns[] = {
 	NULL
 };
 
+/* RESERVED abstract names */
+static const gchar *rabstract_names[] = {
+	"true",
+	"yes",
+	"interface",
+	"mixin",	
+	NULL
+};
+
 static gboolean strv_contains(const char **strv, const xmlChar *str) {
         g_assert(strv != NULL);
         g_assert(str != NULL);
@@ -238,6 +250,23 @@ _get_type_attributes(xmlNode * node, MgdSchemaTypeAttr *type_attr, MidgardSchema
 		
 		}
 
+		/* Abstract  */
+		attrval = xmlGetProp(node, (const xmlChar *)TYPE_RW_ABSTRACT);
+		if (attrval) {
+			if (!strv_contains (rabstract_names, attrval)) {
+				__warn_msg (node, "Invalid abstract type");
+                                g_error ("Invalid abstract type");
+			}
+			if (g_str_equal (attrval, "true")
+					|| g_str_equal (attrval, "yes"))
+				type_attr->is_abstract = TRUE;
+			else if (g_str_equal (attrval, "interface"))
+				type_attr->is_iface = TRUE;
+			else if (g_str_equal (attrval, "mixin")) 
+				type_attr->is_iface = type_attr->is_mixin = TRUE;
+			xmlFree(attrval);			
+		}
+
 		/* parent */
 		attrval = xmlGetProp(node, (const xmlChar *)TYPE_RW_PARENT);
 		if (attrval) {
@@ -252,48 +281,13 @@ _get_type_attributes(xmlNode * node, MgdSchemaTypeAttr *type_attr, MidgardSchema
 		if (attrval) { 
 
 			type_attr->extends = g_strdup ((gchar *)attrval);
-			xmlChar *table_name = (xmlChar *) xmlGetProp (node, (const xmlChar *)"table");	
-
-			if (table_name != NULL) {
-
-				__warn_msg (node, "Can not define 'table' and 'extends' attributes together");
-				xmlFree (table_name);
-
-				/* This might be undefined result, so we fallback to NULL default */
-				g_free (type_attr->extends);
-				type_attr->extends = NULL;
-			
-			} else {
-
-				MgdSchemaTypeAttr *src = midgard_schema_lookup_type (schema, (gchar *)attrval);
-				if (src)
-					midgard_core_schema_type_attr_extend (src, type_attr);
-			}
-
-			xmlFree (table_name);
 			xmlFree (attrval);
 		}
 
 		/* copy */
 		attrval = xmlGetProp(node, (const xmlChar *)TYPE_RW_COPY);
 		if (attrval) {
-
-			type_attr->copy_from = g_strdup((gchar *)attrval);
-			xmlChar *table_name = (xmlChar *)xmlGetProp(node, (const xmlChar *)"table");	
-
-			if (table_name == NULL) {
-
-				__warn_msg(node, "Can not copy MgdSchema type without storage defined");
-				g_error("Table definition missed");
-			
-			} else {
-
-				MgdSchemaTypeAttr *src = midgard_schema_lookup_type (schema, (gchar *)attrval);
-				if (src)
-					midgard_core_schema_type_attr_extend (src, type_attr);
-			}
-	
-			xmlFree(table_name);
+			g_warning ("'copy' attribute is obsolete. Please add 'table' and 'extends' attributes");
 			xmlFree(attrval);
 		}
 
@@ -556,7 +550,15 @@ _get_property_attributes(xmlNode * node,
 		prop_attr->field = g_ascii_strdown ((gchar *)attrval, -1);
 		xmlFree (attrval);
 	} else {
-		prop_attr->field = g_ascii_strdown (prop_attr->name, -1);
+		gchar *dash = g_strstr_len (prop_attr->name, -1, "-");
+		if (dash != NULL) {
+			gchar *tmp = g_strdup (prop_attr->name);
+			prop_attr->field = g_ascii_strdown (g_strdelimit (tmp, "-", '_'), -1);
+			g_free (tmp);
+		}
+		else {
+			prop_attr->field = g_ascii_strdown (prop_attr->name, -1);
+		}
 	}
 		
 	/* index */
@@ -868,6 +870,7 @@ _get_element_names (xmlNode *curn , MgdSchemaTypeAttr *type_attr, MidgardSchema
 
 		type_attr = midgard_core_schema_type_attr_new();
 		type_attr->name = g_strdup((gchar *)nv);
+		type_attr->schema = schema;
 		g_hash_table_insert(schema->types, g_strdup((gchar *)nv), type_attr);
 		_get_type_attributes (obj, type_attr, schema);
 
@@ -949,16 +952,11 @@ void __get_tdata_foreach(gpointer key, gpointer value, gpointer user_data)
 		return;
 	
 	np = g_hash_table_size(type_attr->prophash);
+	if (np < 1)
+		return;
 
-	if (np > 0) {
-
-		if (type_attr->table)
-			g_hash_table_foreach(type_attr->prophash, __get_pdata_foreach, type_attr);
-		
-	} else {
-
-		g_warning("Type %s has less than 1 property!", (gchar *)key);
-	}
+	if (type_attr->table)
+		g_hash_table_foreach(type_attr->prophash, __get_pdata_foreach, type_attr);	
 }
 
 /* Copy source schema types ( class names ) and its MgdSchemaType structures
@@ -1398,34 +1396,75 @@ static void __extend_type_attr(gpointer key, gpointer val, gpointer userdata)
 	midgard_core_schema_type_property_copy(parent_attr, type);
 }
 
+static void __extend_type(MgdSchemaTypeAttr *type_attr, MidgardSchema *schema)
+{
+	if (type_attr->extends == NULL) return;
+
+	MgdSchemaTypeAttr *parent_type_attr = NULL;
+	guint n_types;
+	guint i;
+	gchar **extends = midgard_core_schema_type_list_extends (type_attr, &n_types);
+
+	if (n_types == 0)
+		return;
+
+	for (i = 0; i < n_types; i++) {
+		MgdSchemaTypeAttr *parent_type_attr = midgard_schema_lookup_type(schema, extends[i]);
+		GType parent_type = g_type_from_name (extends[i]);
+
+		/* Try to get type from class' data */
+		if (!parent_type_attr) {
+			GObjectClass *klass = g_type_class_peek (parent_type);
+			if (klass && MIDGARD_IS_DBOBJECT_CLASS(klass)) {
+				parent_type_attr = MIDGARD_DBOBJECT_CLASS (klass)->dbpriv->storage_data;
+			}
+		}
+		
+		if (!parent_type_attr && 
+				(parent_type != G_TYPE_NONE && !G_TYPE_IS_ABSTRACT(parent_type))) {
+			g_error("Type information for %s (%s's parent) not found", 
+					extends[i], type_attr->name);
+		}
+
+		/* We do not need to start inheritance chain, if it's abstract type 
+		 * or type without storage or properties defined */
+		if (!parent_type_attr)
+			return;
+		
+		/* Chain up */	
+		__extend_type (parent_type_attr, schema);
+
+		midgard_core_schema_type_attr_extend (parent_type_attr, type_attr);
+		
+		/* Use parent's storage */
+		if (type_attr->table == NULL)
+			type_attr->table = g_strdup(parent_type_attr->table);
+		if (type_attr->tables == NULL)
+			type_attr->tables = g_strdup(parent_type_attr->tables);
+		
+		GSList *slist = NULL;		
+		for (slist = parent_type_attr->_properties_list; slist != NULL; slist = slist->next) {
+			if (g_slist_find_custom (type_attr->_properties_list, slist->data, (GCompareFunc) g_ascii_strcasecmp) == NULL)
+				type_attr->_properties_list = g_slist_prepend (type_attr->_properties_list, slist->data);
+		}
+		g_hash_table_foreach(parent_type_attr->prophash, __extend_type_attr, type_attr);	
+	}
+
+	g_strfreev (extends);
+}
+
 static void __extend_type_foreach(gpointer key, gpointer val, gpointer userdata)
 {
+	if (val == NULL)
+		return;
+
 	MgdSchemaTypeAttr *type_attr = (MgdSchemaTypeAttr *) val;
 	MidgardSchema *schema = (MidgardSchema*) userdata;
 
-	if (val == NULL) return;
-	if (type_attr->extends == NULL) return;
+	if (type_attr->extends == NULL) 
+		return;
 
-	MgdSchemaTypeAttr *parent_type_attr = 
-		midgard_schema_lookup_type(schema, type_attr->extends);
-
-	if (!parent_type_attr)
-		g_error("Type information for %s (%s's parent) not found", 
-				type_attr->extends, type_attr->name);
-
-	midgard_core_schema_type_attr_extend (type_attr, parent_type_attr);
-
-	/* Use parent's storage */
-	type_attr->table = g_strdup(parent_type_attr->table);
-	type_attr->tables = g_strdup(parent_type_attr->tables);
-
-        GSList *slist = NULL;
-
-	for (slist = parent_type_attr->_properties_list; slist != NULL; slist = slist->next) {
-		type_attr->_properties_list = g_slist_prepend (type_attr->_properties_list, slist->data);
-	}
-
-	g_hash_table_foreach(parent_type_attr->prophash, __extend_type_attr, type_attr);	
+	__extend_type (type_attr, schema);
 }
 
 static void __copy_type_attr(gpointer key, gpointer val, gpointer userdata)
@@ -1474,34 +1513,72 @@ static void __copy_type_foreach(gpointer key, gpointer val, gpointer userdata)
 	g_hash_table_foreach(parent_type_attr->prophash, __copy_type_attr, type_attr);	
 }
 
-static void __register_schema_type (gpointer key, gpointer val, gpointer user_data)
+static void __register_schema_type (MgdSchemaTypeAttr *type_attr)
 {
-	MgdSchemaTypeAttr *type_attr = (MgdSchemaTypeAttr *) val;
+	GType new_type;
+	if (type_attr->is_abstract) {
+		new_type = midgard_type_register_abstract (type_attr, MIDGARD_TYPE_BASE_ABSTRACT);
+	} else if (type_attr->is_mixin 
+			|| type_attr->is_iface) {
+		new_type = midgard_core_type_register_interface (type_attr);
+		return;
+	} else {
+		new_type = midgard_type_register(type_attr);
+	}
 
-	if (val == NULL) return;
-	if (type_attr->params == NULL) {
-
-		g_warning("No parameters found for %s schema type. Not registering.", type_attr->name);
+	if (!type_attr->is_abstract && g_type_is_a (new_type, MIDGARD_TYPE_BASE_ABSTRACT)) 
+	{
+		g_warning ("Can not register '%s' type, which is derived from '%s' abstract template", 
+				type_attr->name, g_type_name (g_type_parent (new_type)));
 		return;
 	}
 
-	GType new_type;
-	new_type = midgard_type_register(type_attr, MIDGARD_TYPE_OBJECT);
-
 	if (new_type) {
 
-		GObject *foo = g_object_new(new_type, NULL);
+		/* Create new instance to ensure class is initialized */
+		if (!G_TYPE_IS_ABSTRACT (new_type)) {
+			GObject *foo = g_object_new(new_type, NULL);
+			g_object_unref (foo);
+		}
+		GObjectClass *klass = g_type_class_peek (new_type);
+		if (!klass)
+			klass = g_type_class_ref (new_type);
 		/* Set number of properties.
 		 * This way we gain performance for instance_init call */
-		GParamSpec **pspecs = 
-			g_object_class_list_properties(
-					G_OBJECT_GET_CLASS(G_OBJECT(foo)), 
-					&type_attr->class_nprop);
+		GParamSpec **pspecs = g_object_class_list_properties (klass, &type_attr->class_nprop);
 		g_free(pspecs);
-		g_object_unref(foo); 
 	}
 }
 
+static void __register_schema_type_foreach (gpointer key, gpointer val, gpointer user_data)
+{
+	if (val == NULL)
+		return;
+	__register_schema_type ((MgdSchemaTypeAttr *) val);
+}
+
+static void __get_interfaces (gpointer key, gpointer val, gpointer user_data)
+{
+	GSList **ifaces = (GSList **) user_data;
+	MgdSchemaTypeAttr *type_attr = (MgdSchemaTypeAttr *) val;
+
+	if (type_attr->is_iface 
+			|| type_attr->is_mixin)
+		*ifaces = g_slist_append (*ifaces, type_attr);
+}
+
+static void __midgard_schema_register_interfaces (MidgardSchema *self)
+{
+	GSList *ifaces = NULL;
+	g_hash_table_foreach (self->types, __get_interfaces, &ifaces);
+
+	GSList *l = NULL;
+	for (l = ifaces; l != NULL; l = l->next) {	
+		__register_schema_type ((MgdSchemaTypeAttr *) l->data);
+	}
+	g_slist_free (ifaces);
+}
+
 static void __build_static_sql (gpointer key, gpointer val, gpointer userdata)
 {
 	MgdSchemaTypeAttr *type_attr = (MgdSchemaTypeAttr *)val;
@@ -1530,7 +1607,8 @@ static void __midgard_schema_postconfig(MidgardSchema *self)
 	g_hash_table_foreach (self->types, __initialize_paramspec, NULL);
 	g_hash_table_foreach (self->types, __build_static_sql, NULL);
 	g_hash_table_foreach (self->types, __validate_fields, NULL);
-	g_hash_table_foreach (self->types, __register_schema_type, NULL);
+	__midgard_schema_register_interfaces (self);
+	g_hash_table_foreach (self->types, __register_schema_type_foreach, NULL);
 }
 
 /**
