@@ -172,8 +172,6 @@ static void __set_config_from_keyfile(MidgardConfig *self, GKeyFile *keyfile, co
 	guint port = g_key_file_get_integer (keyfile, "MidgardDatabase", "Port", NULL);
 	if (port > 0)
 		self->dbport = port;
-	else if (port < 0) 
-		g_warning ("Invalid, negative value for database port");
 
 	/* Get database name */
 	tmpstr = g_key_file_get_string (keyfile, "MidgardDatabase", "Name", NULL);
@@ -213,6 +211,7 @@ static void __set_config_from_keyfile(MidgardConfig *self, GKeyFile *keyfile, co
 
 		__create_log_dir((const gchar *) tmpstr);
 
+		g_free (self->logfilename);
 		self->logfilename = g_strdup(tmpstr);
 		GError *err = NULL;
 		GFile *g_file = g_file_new_for_path ((const gchar *)self->logfilename);
@@ -525,7 +524,7 @@ gboolean midgard_config_read_data(MidgardConfig *self, const gchar *data, GError
  *
  * Returned array should be freed when no longer needed.
  *
- * Returns: newly allocated and %NULL terminated array of file names.
+ * Returns: (transfer full): newly allocated and %NULL terminated array of file names.
  *
  */
 gchar **midgard_config_list_files(gboolean user)
@@ -552,37 +551,40 @@ gchar **midgard_config_list_files(gboolean user)
 
 	GError *error = NULL;
 	GSList *list = NULL;
+	GSList *l = NULL;
 	gchar **filenames = NULL;
-	guint i = 0, j = 0;
+	guint i = 0;
 	GDir *dir = g_dir_open(config_dir, 0, &error);
-	if (dir != NULL) {
-		const gchar *file = g_dir_read_name(dir);
-	
-		while (file != NULL) {
-			
-			if(!g_str_has_prefix(file, ".")
-					&& (!g_str_has_prefix(file, "#"))
-					&& (!g_str_has_suffix(file, "~"))
-					&& (!g_str_has_suffix(file, "example"))) {
-					
-				list = g_slist_append(list, (gpointer)g_strdup(file));
-				i++;
-			}
-			file = g_dir_read_name(dir);
-		}
-		
-		g_dir_close(dir);
-		
-		filenames = g_new(gchar *, i+1);
-		for( ; list; list = list->next) {
-			filenames[j] = (gchar *)list->data;
-			j++;
-		}
-		filenames[i] = NULL;		
-	}
-	
+
 	g_free (config_dir);
 
+	if (dir == NULL) 
+		return filenames;
+	
+	const gchar *file = g_dir_read_name(dir);
+	while (file != NULL) {
+		if(!g_str_has_prefix(file, ".")
+				&& (!g_str_has_prefix(file, "#"))
+				&& (!g_str_has_suffix(file, "~"))
+				&& (!g_str_has_suffix(file, "example"))) {
+			list = g_slist_append(list, (gpointer)g_strdup(file));
+			i++;
+		}
+		file = g_dir_read_name(dir);
+	}
+
+	g_dir_close(dir);
+	
+	filenames = g_new(gchar *, i+1);
+
+	i = 0;
+	for(l = list; l != NULL; l = l->next, i++) 
+		filenames[i] = (gchar *)l->data;
+
+	filenames[i] = NULL;		
+	
+	g_slist_free (list);
+
 	return filenames;
 }
 
@@ -1395,7 +1397,7 @@ void midgard_config_set_pamfile(MidgardConfig *self, const gchar *pamfile)
  * midgard_config_copy:
  * @self: #MidgardConfig instance
  *
- * Returns: deep copy of given #MidgardConfig object
+ * Returns: (transfer full): deep copy of given #MidgardConfig object
  * Since: 10.05
  */ 
 MidgardConfig *
