@@ -49,13 +49,13 @@ struct _MidgardDBJoin {
 	gchar *type;
 	guint typeid;
 	gchar *table;
-	MidgardObjectClass *klass;
+	MidgardDBObjectClass *klass;
 	gchar *left_table;
 	gchar *left_field;
-	MidgardObjectClass *left_klass;
+	MidgardDBObjectClass *left_klass;
 	gchar *right_table;
 	gchar *right_field;
-	MidgardObjectClass *right_klass;
+	MidgardDBObjectClass *right_klass;
 	gchar *left_tablefield;
 	gchar *right_tablefield;
 
@@ -74,12 +74,14 @@ struct _MidgardQueryStoragePrivate {
 	gchar *table_alias;
 	const gchar *table;
 	const gchar *classname;
+	gboolean is_valid;
 };
 
 struct _MidgardQueryPropertyPrivate {
 	GValue value;
 	MidgardDBObjectClass *klass;
 	MidgardQueryStorage *storage;
+	gboolean is_valid;
 };
 
 struct _MidgardQueryExecutorPrivate {
@@ -99,11 +101,12 @@ struct _MidgardQueryExecutorPrivate {
 	guint results_count;
 	gboolean read_only;
 	gboolean include_deleted;
+	gboolean is_valid;
 };
 
 struct _MidgardQueryConstraintSimplePrivate {
 	void	(*add_conditions_to_statement)	(MidgardQueryExecutor *executor, MidgardQueryConstraintSimple *self, 
-			GdaSqlStatement *stm, GdaSqlExpr *where_expr_node);
+			GdaSqlStatement *stm, GdaSqlExpr *where_expr_node, GError **error);
 };
 
 #define MQE_SET_TABLE_ALIAS(__executor, __storage) \
@@ -166,6 +169,8 @@ void		midgard_core_dbjoin_free	(MidgardDBJoin *mdbj);
 	if (__value == NULL) { \
 		g_warning ("Invalid value (NULL). Converting to default FALSE boolean value"); \
 		__prop = FALSE; \
+	} else if(G_VALUE_HOLDS_BOOLEAN(__value)) { \
+		__prop = g_value_get_boolean(__value); \
 	} else if(G_VALUE_HOLDS_UINT(__value)) { \
 		guint __i = g_value_get_uint(__value); \
 		if(__i == 1) \
@@ -227,6 +232,11 @@ GValue *midgard_core_query_get_field_value(
 			const gchar *table, 
 			const gchar *where);
 
+gboolean midgard_core_query_get_object_value(
+			MidgardDBObject *dbobject,
+			const gchar *column,
+			GValue *value);
+
 guint midgard_core_query_get_id(
 			MidgardConnection *mgd, 
 			const gchar *table,
@@ -282,9 +292,11 @@ gboolean midgard_core_table_exists(
 
 GdaDataModel 		*midgard_core_query_get_dbobject_model 		(MidgardConnection *mgd, MidgardDBObjectClass *klass, guint n_params, const GParameter *parameters);
 gboolean		midgard_core_query_create_dbobject_record 	(MidgardDBObject *object);
-gboolean		midgard_core_query_update_dbobject_record 	(MidgardDBObject *object);
+gint			midgard_core_query_update_dbobject_record 	(MidgardDBObject *object, GError **error);
 gchar                   *midgard_core_query_binary_stringify            (GValue *src_value); 
-gchar 			*midgard_core_query_compute_constraint_property	(MidgardQueryExecutor *executor, MidgardQueryStorage *storage, const gchar *name);
+gchar 			*midgard_core_query_compute_constraint_property	(MidgardQueryExecutor *executor, MidgardQueryStorage *storage, const gchar *name, GError **error);
 gchar 			*midgard_core_query_unescape_string (MidgardConnection *mgd, const gchar *str);
+void			midgard_core_query_get_object (MidgardConnection *mgd, const gchar *classname, MidgardDBObject **object, GError **error, const gchar *property, ...);
+MidgardDBObject 	**midgard_core_query_get_objects (MidgardConnection *mgd, const gchar *classname, GError **error, const gchar *property, ...);
 
 #endif /* MIDGARD_CORE_QUERY_H */
