@@ -23,6 +23,7 @@
 #include "guid.h"
 #include "midgard_core_object_parameter.h"
 #include "midgard_blob.h"
+#include "midgard_core_query.h"
 
 static MidgardCollector *__create_domain_collector(MidgardConnection *mgd, const gchar *domain)
 {
@@ -314,7 +315,7 @@ midgard_object_set_parameter (MidgardObject *self, const gchar *domain, const gc
  *
  * Returned array should be freed when no longer needed.
  * 
- * Returns: Newly allocated and NULL terminated array of midgard_parameter objects. 
+ * Returns: (transfer full): Newly allocated and NULL terminated array of midgard_parameter objects. 
  */
 MidgardObject **midgard_object_list_parameters(MidgardObject *self, const gchar *domain)
 {
@@ -414,7 +415,7 @@ gboolean midgard_object_purge_parameters(MidgardObject *self,
  * @parameters argument is optional. All object's parameters are 
  * returned ( if exist ) if @parameters is explicitly set to %NULL.
 *
- * Returns: newly created, NULL terminated array of #MidgardObject ( midgard_parameter class ) or %NULL on failure
+ * Returns: (transfer full): newly created, NULL terminated array of #MidgardObject ( midgard_parameter class ) or %NULL on failure
  */
 MidgardObject **midgard_object_find_parameters(MidgardObject *self, 
 		guint n_params, const GParameter *parameters)
@@ -453,24 +454,21 @@ MidgardObject **midgard_core_object_parameters_list(
 	g_assert(class_name != NULL);
 	g_assert(guid != NULL);
 
-	MidgardQueryBuilder *builder =
-		midgard_query_builder_new(mgd, class_name);
-
-	if(!builder)
-		return NULL;
-
 	GValue gval = {0, };
 	g_value_init(&gval, G_TYPE_STRING);
 	g_value_set_string(&gval, guid);
-	midgard_query_builder_add_constraint(builder,
-			"parentguid", "=", &gval);
+	GError *err = NULL;
+	
+	MidgardDBObject **objects = midgard_core_query_get_objects (mgd, class_name, &err, "parentguid", &gval, NULL);
+
 	g_value_unset(&gval);
-
-	guint n_objects;	
-	GObject **objects =
-		midgard_query_builder_execute(builder, &n_objects);
-
-	g_object_unref(builder);
+	
+	if (err) {
+		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_NOT_EXISTS, "%s",
+				err && err->message ? err->message : "Unknown reason");
+		g_clear_error (&err);
+		return NULL;
+	}
 
 	return (MidgardObject **)objects;
 }
