@@ -49,7 +49,6 @@ struct _MidgardQueryExecutorClass {
 	gboolean	(*add_order)			(MidgardQueryExecutor *self, MidgardQueryProperty *property, const gchar *type);
 	gboolean        (*add_join)			(MidgardQueryExecutor *self, const gchar *join_type, 
 								MidgardQueryProperty *left_property, MidgardQueryProperty *right_property);
-	gboolean	(*execute)			(MidgardQueryExecutor *self);
 	guint 		(*get_results_count)		(MidgardQueryExecutor *self);
 
 	/* signals */
@@ -75,7 +74,6 @@ gboolean		midgard_query_executor_set_limit	(MidgardQueryExecutor *self, guint li
 gboolean 		midgard_query_executor_set_offset	(MidgardQueryExecutor *self, guint offset);
 gboolean		midgard_query_executor_add_order	(MidgardQueryExecutor *self, MidgardQueryProperty *property, const gchar *type);
 gboolean		midgard_query_executor_add_join		(MidgardQueryExecutor *self, const gchar *join_type, MidgardQueryProperty *left_property, MidgardQueryProperty *right_property);
-gboolean		midgard_query_executor_execute		(MidgardQueryExecutor *self);
 guint 			midgard_query_executor_get_results_count	(MidgardQueryExecutor *self);
 
 G_END_DECLS
