@@ -22,6 +22,11 @@
 #include "midgard_core_object_class.h"
 #include "midgard_query_holder.h"
 #include "midgard_dbobject.h"
+#include "midgard_validable.h"
+#include "midgard_executable.h"
+#include <sql-parser/gda-sql-parser.h>
+#include "midgard_core_workspace.h"
+#include "midgard_workspace_storage.h"
 
 /**
  * midgard_query_select_new:
@@ -30,6 +35,9 @@
  *
  * #MidgardStorage @storage represents storage which is queried during execution
  *
+ * By default, for performance reason, MidgardQuerySelect is instantiated in read only mode.
+ * See: midgard_query_select_read_only().
+ *
  * Returns: new #MidgardQuerySelect instance or %NULL on failure
  * Since: 10.05
  */ 
@@ -63,7 +71,6 @@ gboolean
 _midgard_query_select_set_limit (MidgardQueryExecutor *self, guint limit)
 {
 	g_return_val_if_fail (self != NULL, FALSE);
-	g_return_val_if_fail (limit > 0, FALSE);
 
 	self->priv->limit = limit;
 
@@ -74,11 +81,6 @@ gboolean
 _midgard_query_select_set_offset (MidgardQueryExecutor *self, guint offset)
 {
 	g_return_val_if_fail (self != NULL, FALSE);
-	if (offset < 0) {
-		g_warning ("Negative offset value not allowed. Setting default value 0");
-		offset = 0;
-	}
-
 	self->priv->offset = offset;
 
 	return TRUE;
@@ -87,6 +89,7 @@ _midgard_query_select_set_offset (MidgardQueryExecutor *self, guint offset)
 typedef struct {
 	MidgardQueryProperty *property;
 	gboolean asc;
+	gchar *order_type;
 } qso;
 
 gboolean
@@ -95,21 +98,9 @@ _midgard_query_select_add_order (MidgardQueryExecutor *self, MidgardQueryPropert
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (property != NULL, FALSE);
 
-	gboolean asc = FALSE;
-
-	gchar *lorder = g_ascii_strdown (type, -1);
-	if (g_str_equal (lorder, "asc")) {
-		asc = TRUE;
-	} else if (g_str_equal (lorder, "desc")) {
-		asc = FALSE;
-	} else {
-		g_warning ("Invalid order type '%s'. Expected ASC or DESC", type);
-	}
-
-	g_free (lorder);
-
 	qso *_qs = g_new (qso, 1);
-	_qs->asc = asc;
+	_qs->order_type = g_strdup (type);
+	_qs->asc = FALSE;
 	_qs->property = g_object_ref (property);
 
 	self->priv->orders = g_slist_append (self->priv->orders, _qs);
@@ -183,8 +174,8 @@ _midgard_query_select_add_join (MidgardQueryExecutor *self, const gchar *join_ty
 	}
 
 	qsj *_sj = g_new (qsj, 1);
-	_sj->left_property = left_property;
-	_sj->right_property = right_property;
+	_sj->left_property = g_object_ref (left_property);
+	_sj->right_property = g_object_ref (right_property);
 
 	_sj->join_type = join_type_id;
 
@@ -193,7 +184,7 @@ _midgard_query_select_add_join (MidgardQueryExecutor *self, const gchar *join_ty
 	return TRUE;
 }
 
-gboolean __query_select_add_orders (MidgardQueryExecutor *self)
+gboolean __query_select_add_orders (MidgardQueryExecutor *self, GError **error)
 {
 	if (!self->priv->orders)
 		return TRUE;
@@ -207,6 +198,7 @@ gboolean __query_select_add_orders (MidgardQueryExecutor *self)
 	
 	for (l = MIDGARD_QUERY_EXECUTOR (self)->priv->orders; l != NULL; l = l->next) {
 
+		/* Proper asc type is set during validation phase */
 		qso *_so = (qso*) l->data;
 
 		/* Create new order */
@@ -223,7 +215,22 @@ gboolean __query_select_add_orders (MidgardQueryExecutor *self)
 		/* Compute table.colname for given property name */
 		GValue rval = {0, };
 		midgard_query_holder_get_value (MIDGARD_QUERY_HOLDER (property), &rval);
-		gchar *table_field = midgard_core_query_compute_constraint_property (executor, storage, g_value_get_string (&rval));
+		GError *err = NULL;
+		gchar *table_field = midgard_core_query_compute_constraint_property (executor, storage, g_value_get_string (&rval), &err);
+		if (err) {
+			g_propagate_error (error, err);
+			g_free (table_field);
+			gda_sql_select_order_free (order);
+			return FALSE;
+		}
+
+		if (!table_field) {
+			g_set_error (error, MIDGARD_VALIDATION_ERROR, MIDGARD_VALIDATION_ERROR_LOCATION_INVALID,
+					"Can not find table and column name for given '%s' property name", g_value_get_string (&rval));
+			g_value_unset (&rval);
+			return FALSE;	
+		}
+
 		g_value_unset (&rval);
 
 		GValue *value = g_new0 (GValue, 1);
@@ -240,8 +247,102 @@ gboolean __query_select_add_orders (MidgardQueryExecutor *self)
 	return TRUE;
 }
 
-gboolean __query_select_add_joins (MidgardQuerySelect *self)
+static void
+__add_implicit_workspace_join (MidgardQuerySelect *self, GdaSqlOperation *operation)
 {
+	g_return_if_fail (self != NULL);
+	
+	MidgardDBObjectClass *klass = MIDGARD_QUERY_EXECUTOR (self)->priv->storage->priv->klass;
+	if (!g_type_is_a (G_OBJECT_CLASS_TYPE (klass), MIDGARD_TYPE_OBJECT))
+		return;
+
+	MidgardConnection *mgd = MIDGARD_QUERY_EXECUTOR (self)->priv->mgd;
+	/* Do not take worskpace into account if if it's disabled 
+	 * or enabled and none set */
+	if (!MGD_CNC_USES_WORKSPACE (mgd))
+		return;
+	if (!MGD_CNC_HAS_WORKSPACE (mgd))
+		return;
+
+	guint ws_id = MGD_CNC_WORKSPACE_ID (mgd);
+	MidgardQueryExecutor *executor = MIDGARD_QUERY_EXECUTOR (self);
+	GdaSqlStatement *sql_stm = executor->priv->stmt;
+	GdaSqlStatementSelect *select = (GdaSqlStatementSelect *) sql_stm->contents;
+	GdaSqlSelectFrom *from = select->from;
+	GdaSqlSelectJoin *join;
+	const gchar *klass_table = MGD_DBCLASS_TABLENAME (klass);
+
+	gchar *left_table = executor->priv->table_alias;
+	gchar *right_table = g_strdup_printf ("t%d", ++executor->priv->tableid);
+
+	join = gda_sql_select_join_new (GDA_SQL_ANY_PART (from));
+	join->type = GDA_SQL_SELECT_JOIN_INNER;
+
+	GdaSqlExpr *expr = gda_sql_expr_new (GDA_SQL_ANY_PART (join));
+	expr->value = gda_value_new (G_TYPE_STRING);
+	g_value_take_string (expr->value, g_strdup_printf ("%s.%s = %s.%s", 
+				left_table, MGD_WORKSPACE_ID_FIELD, right_table, MGD_WORKSPACE_ID_FIELD));
+
+	join->expr = expr;
+	join->position = ++executor->priv->joinid;
+
+	const gchar *deleted_field = midgard_core_object_get_deleted_field (klass);
+	GString *table = g_string_new ("(SELECT DISTINCT MAX");
+	g_string_append_printf (table, "(%s) AS %s, %s %s%s FROM %s WHERE %s IN (0,", 
+			MGD_WORKSPACE_ID_FIELD, MGD_WORKSPACE_ID_FIELD, MGD_WORKSPACE_OID_FIELD, 
+			deleted_field ? "," : "",
+			deleted_field ? deleted_field : "",
+			klass_table, MGD_WORKSPACE_ID_FIELD);
+
+	const MidgardWorkspaceStorage *ws = midgard_connection_get_workspace (mgd);
+	GSList *list = MIDGARD_WORKSPACE_STORAGE_GET_INTERFACE (ws)->priv->list_ids (mgd, MIDGARD_WORKSPACE_STORAGE (ws));
+	GSList *l = NULL;
+	guint i = 0;
+	guint id;
+	for (l = list; l != NULL; l = l->next, i++) {
+		GValue *id_val = (GValue *) l->data;
+		if (G_VALUE_HOLDS_UINT (id_val))
+			id = g_value_get_uint (id_val);
+		else
+			id = (guint) g_value_get_int (id_val);
+		g_string_append_printf (table, "%s%d", i > 0 ? "," : "", id);
+	}
+	if (!list) 
+		g_string_append (table, "0");
+	g_string_append (table, ") ");  
+	if (deleted_field)
+		g_string_append_printf (table, "AND %s = 0 ", deleted_field);
+	g_string_append_printf (table, "GROUP BY %s)", MGD_WORKSPACE_OID_FIELD);
+	g_slist_free (list);
+
+	gda_sql_select_from_take_new_join (from , join);
+	GdaSqlSelectTarget *s_target = gda_sql_select_target_new (GDA_SQL_ANY_PART (from));
+	s_target->table_name = g_string_free (table, FALSE);
+	s_target->as = right_table;
+	gda_sql_select_from_take_new_target (from, s_target);
+	/* MIDGARD_QUERY_EXECUTOR (self)->priv->include_deleted_targets = 
+		g_slist_append (MIDGARD_QUERY_EXECUTOR (self)->priv->include_deleted_targets, s_target); */
+
+	GdaSqlExpr *texpr = gda_sql_expr_new (GDA_SQL_ANY_PART (s_target));
+	GValue *tval = g_new0 (GValue, 1);
+	g_value_init (tval, G_TYPE_STRING);
+	g_value_set_string (tval, g_strdup (s_target->table_name));
+	texpr->value = tval;
+	s_target->expr = texpr;
+
+	/* Add workspace object id constraint */
+	GdaSqlExpr *ws_expr = gda_sql_expr_new (GDA_SQL_ANY_PART (operation));
+	ws_expr->value = gda_value_new (G_TYPE_STRING);
+	g_value_take_string (ws_expr->value, g_strdup_printf ("%s.%s = %s.%s", 
+				left_table, MGD_WORKSPACE_OID_FIELD, right_table, MGD_WORKSPACE_OID_FIELD));
+	operation->operands = g_slist_append (operation->operands, ws_expr);
+
+}
+
+gboolean __query_select_add_joins (MidgardQuerySelect *self, GdaSqlOperation *operation, GError **error)
+{
+	__add_implicit_workspace_join (self, operation);
+
 	if (!MIDGARD_QUERY_EXECUTOR (self)->priv->joins)
 		return TRUE;
 
@@ -263,15 +364,24 @@ gboolean __query_select_add_joins (MidgardQuerySelect *self)
 		MidgardQueryStorage *left_storage = _sj->left_property->priv->storage;
 		MidgardQueryStorage *right_storage = _sj->right_property->priv->storage;
 
+		GError *err = NULL;
 		GValue lval = {0, };
 		midgard_query_holder_get_value (MIDGARD_QUERY_HOLDER (_sj->left_property), &lval);
 		gchar *left_table_field = 
-			midgard_core_query_compute_constraint_property (executor, left_storage, g_value_get_string (&lval));
+			midgard_core_query_compute_constraint_property (executor, left_storage, g_value_get_string (&lval), &err);
+		if (err) {
+			g_propagate_error (error, err);
+			g_free (left_table_field);
+		}
 
 		GValue rval = {0, };
 		midgard_query_holder_get_value (MIDGARD_QUERY_HOLDER (_sj->right_property), &rval);
 		gchar *right_table_field = 
-			midgard_core_query_compute_constraint_property (executor, right_storage, g_value_get_string (&rval));
+			midgard_core_query_compute_constraint_property (executor, right_storage, g_value_get_string (&rval), &err);
+		if (err) {
+			g_propagate_error (error, err);
+			g_free (right_table_field);
+		}
 
 		GdaSqlExpr *expr = gda_sql_expr_new (GDA_SQL_ANY_PART (join));
 		expr->value = gda_value_new (G_TYPE_STRING);
@@ -361,31 +471,138 @@ __add_second_dummy_constraint (GdaSqlStatementSelect *select, GdaSqlOperation *t
 	return;
 }
 
-gboolean 
-_midgard_query_select_execute (MidgardQueryExecutor *self)
+static void
+_get_all_constraints (MidgardQuerySelect *self, MidgardQueryConstraintSimple *constr, GSList **list)
+{
+	if (!constr)
+		if (self && MIDGARD_QUERY_EXECUTOR (self)->priv->constraint)
+			constr = MIDGARD_QUERY_EXECUTOR (self)->priv->constraint;
+
+	if (!constr)
+		return;
+
+	*list = g_slist_prepend (*list, constr);
+	guint n_objects;
+	MidgardQueryConstraintSimple **constraints = midgard_query_constraint_simple_list_constraints (constr, &n_objects);
+	if (!constraints)
+		return;
+
+	guint i;
+	for (i = 0; i < n_objects; i++) {
+		_get_all_constraints (NULL, constraints[i], list);
+	}
+
+	g_free (constraints);
+}
+
+static void
+_midgard_query_select_validable_iface_validate (MidgardValidable *iface, GError **error)
+{
+	g_return_if_fail (iface != NULL);
+	MidgardQuerySelect *self = MIDGARD_QUERY_SELECT (iface);
+	MIDGARD_QUERY_EXECUTOR (self)->priv->is_valid = FALSE;
+
+	GError *err = NULL;
+	MidgardQueryStorage *storage = MIDGARD_QUERY_EXECUTOR (self)->priv->storage;
+	GObjectClass *klass = MIDGARD_QUERY_EXECUTOR (self)->priv->storage->priv->klass;
+	const gchar *table = midgard_core_class_get_table (klass);
+
+	/* Class table */
+	if (!table) {
+		g_set_error (error, MIDGARD_VALIDATION_ERROR, MIDGARD_VALIDATION_ERROR_LOCATION_INVALID,
+				"No SQL table defined for '%s' class", G_OBJECT_CLASS_NAME (klass));
+		return;
+	}
+
+	/* Storage */
+	if (!storage) {
+		g_set_error (error, MIDGARD_VALIDATION_ERROR, MIDGARD_VALIDATION_ERROR_LOCATION_INVALID,
+				"No QueryStorage associated with QuerySelect");
+		return;
+	}
+
+	if (storage->priv->klass && !storage->priv->klass->dbpriv->add_fields_to_select_statement) {
+		g_set_error (error, MIDGARD_VALIDATION_ERROR, MIDGARD_VALIDATION_ERROR_INTERNAL,
+				"'%s' DBObjectClass doesn't provide select statement.", G_OBJECT_CLASS_NAME (storage->priv->klass));
+		return;
+	}
+
+	/* Limit */
+	if (MIDGARD_QUERY_EXECUTOR (self)->priv->limit == 0) {
+		g_set_error (error, MIDGARD_VALIDATION_ERROR, MIDGARD_VALIDATION_ERROR_VALUE_INVALID,
+				"Invalid limit value (0). Expected > 0");
+		return;
+	}
+
+	/* Orders */
+	GSList *l = NULL;
+	for (l = MIDGARD_QUERY_EXECUTOR (self)->priv->orders; l != NULL; l = l->next) {
+		qso *_so = (qso*) l->data;
+		gboolean asc = FALSE;
+		gchar *lorder = g_ascii_strdown (_so->order_type, -1);
+		if (g_str_equal (lorder, "asc")) {
+			_so->asc = TRUE;
+		} else if (g_str_equal (lorder, "desc")) {
+			_so->asc = FALSE;
+		} else {
+			g_set_error (error, MIDGARD_VALIDATION_ERROR, MIDGARD_VALIDATION_ERROR_VALUE_INVALID,
+					"Invalid order type '%s'. Expected ASC or DESC", _so->order_type);
+			g_free (lorder); 
+			return;
+		}		
+		g_free (lorder);
+	}
+
+	/* Gather all objects required for execution */
+	GSList *o_list = NULL;
+	o_list = g_slist_prepend (o_list, storage);
+	_get_all_constraints (self, NULL, &o_list);
+
+	for (l = o_list; l != NULL; l = l->next) {
+		if (!midgard_validable_is_valid (MIDGARD_VALIDABLE (l->data))) {
+			midgard_validable_validate (MIDGARD_VALIDABLE (l->data), &err);
+			if (err) {
+				g_propagate_error (error, err);
+				g_slist_free (o_list);
+				return;
+			}
+		}
+	}
+
+	if (o_list)
+		g_slist_free (o_list);
+
+	MIDGARD_QUERY_EXECUTOR (self)->priv->is_valid = TRUE;
+	return;
+}
+
+gboolean
+_midgard_query_select_validable_iface_is_valid (MidgardValidable *self)
 {
 	g_return_val_if_fail (self != NULL, FALSE);
-	
-	GError *error = NULL;
+	return MIDGARD_QUERY_EXECUTOR (self)->priv->is_valid;
+}
 
-	if (!MIDGARD_QUERY_EXECUTOR (self)->priv->storage) {
-		/* FIXME, handle error */
-		g_warning ("Missed QueryStorage associated with QuerySelect");
-		return FALSE;
-	}
-	
-	MidgardDBObjectClass *klass = self->priv->storage->priv->klass;
-	if (!klass->dbpriv->add_fields_to_select_statement) {
-		/* FIXME, handle error */
-		g_warning ("Missed private DBObjectClass' fields to statement helper");
-		return FALSE;
+static void
+_midgard_query_select_executable_iface_execute (MidgardExecutable *iface, GError **error)
+{
+	g_return_if_fail (iface != NULL);
+	MidgardQuerySelect *self = MIDGARD_QUERY_SELECT (iface);
+	MidgardQueryExecutor *executor = MIDGARD_QUERY_EXECUTOR (self);
+
+	g_signal_emit (self, MIDGARD_QUERY_EXECUTOR_GET_CLASS (self)->signal_id_execution_start, 0);
+
+	GError *err = NULL;
+	midgard_validable_validate (MIDGARD_VALIDABLE (self), &err);
+	if (err) {
+		g_propagate_error (error, err);
+		return;
 	}
 
 	g_object_ref (self);
 
-	g_signal_emit (self, MIDGARD_QUERY_EXECUTOR_GET_CLASS (self)->signal_id_execution_start, 0);
-
-	MidgardConnection *mgd = self->priv->mgd;
+	MidgardDBObjectClass *klass = executor->priv->storage->priv->klass;
+	MidgardConnection *mgd = executor->priv->mgd;
 	GdaConnection *cnc = mgd->priv->connection;
 	GdaSqlStatement *sql_stm;
 	GdaSqlStatementSelect *sss;
@@ -419,20 +636,29 @@ _midgard_query_select_execute (MidgardQueryExecutor *self)
 	s_target->expr = texpr;
 
 	/* Add fields for all properties registered per class (SELECT a,b,c...) */
-	klass->dbpriv->add_fields_to_select_statement (klass, cnc, sss, s_target->as);
-
-	/* Add joins, LEFT JOIN tbl2 ON... */
-	if (!__query_select_add_joins (MIDGARD_QUERY_SELECT (self))) 
-		goto return_false;
+	klass->dbpriv->add_fields_to_select_statement (klass, mgd, sss, s_target->as);
 
 	GdaSqlExpr *where = sss->where_cond;
 	GdaSqlOperation *operation = where->cond;
+	/* Add joins, LEFT JOIN tbl2 ON... */
+	__query_select_add_joins (MIDGARD_QUERY_SELECT (self), operation, &err);
+	if (err) {
+	 	g_propagate_error (error, err);
+		goto return_false;
+	}
 
 	/* Add constraints' conditions (WHERE a=1, b=2...) */
 	if (MIDGARD_QUERY_EXECUTOR (self)->priv->constraint) {
-		MIDGARD_QUERY_CONSTRAINT_SIMPLE_GET_INTERFACE (MIDGARD_QUERY_EXECUTOR (self)->priv->constraint)->priv->add_conditions_to_statement 			(MIDGARD_QUERY_EXECUTOR (self), MIDGARD_QUERY_EXECUTOR (self)->priv->constraint, sql_stm, base_where);
-		if (MIDGARD_QUERY_EXECUTOR (self)->priv->n_constraints == 1)
+		MIDGARD_QUERY_CONSTRAINT_SIMPLE_GET_INTERFACE (MIDGARD_QUERY_EXECUTOR (self)->priv->constraint)->priv->add_conditions_to_statement 			(MIDGARD_QUERY_EXECUTOR (self), MIDGARD_QUERY_EXECUTOR (self)->priv->constraint, sql_stm, base_where, &err);
+		if (err) {
+			g_set_error(error, err->domain, err->code, "%s", err->message ? err->message : "Unknown error");
+			g_clear_error(&err);
+		}
+		if (MIDGARD_QUERY_EXECUTOR (self)->priv->n_constraints == 1) 
 			__add_second_dummy_constraint (sss, operation);
+		/* Add dummy constraint if operation has only one operand */
+		if (operation->operands && (g_slist_length (operation->operands) == 1))
+			__add_dummy_constraint (sss, operation);
 	} else { 
 		/* no constraints, add dummy '1=1 AND 0<1' to satisfy top constraint group */
 		__add_dummy_constraint (sss, operation); 
@@ -440,39 +666,42 @@ _midgard_query_select_execute (MidgardQueryExecutor *self)
 	}
 
 	/* Add orders , ORDER BY t1.field... */
-	if (!__query_select_add_orders (self)) 
+	if (!__query_select_add_orders (executor, &err)) { 
+		if (err)
+			g_propagate_error (error, err);
 		goto return_false;
+	}
 
 	/* Exclude deleted */
-	if (MGD_DBCLASS_METADATA_CLASS (klass) && !MIDGARD_QUERY_EXECUTOR (self)->priv->include_deleted)
+	if (MGD_DBCLASS_METADATA_CLASS (klass) && !executor->priv->include_deleted)
 		__add_exclude_deleted_constraints (sss, operation, klass);
 
 	/* Add limit, LIMIT x */
-	if (MIDGARD_QUERY_EXECUTOR (self)->priv->limit > 0) {
+	if (executor->priv->limit > 0) {
 		GdaSqlExpr *limit_expr = gda_sql_expr_new (GDA_SQL_ANY_PART (sss));
 		GValue *limit_val = g_new0 (GValue, 1);
 		g_value_init (limit_val, G_TYPE_STRING);
-		g_value_take_string (limit_val, g_strdup_printf ("%d", MIDGARD_QUERY_EXECUTOR (self)->priv->limit));
+		g_value_take_string (limit_val, g_strdup_printf ("%d", executor->priv->limit));
 		limit_expr->value = limit_val;
 		sss->limit_count = limit_expr;
 	}
 
 	/* Add offset, OFFSET x */
-	if (MIDGARD_QUERY_EXECUTOR (self)->priv->offset >= 0) {
+	if (executor->priv->offset >= 0) {
 		GdaSqlExpr *offset_expr = gda_sql_expr_new (GDA_SQL_ANY_PART (sss));
 		GValue *offset_val = g_new0 (GValue, 1);
 		g_value_init (offset_val, G_TYPE_STRING);
-		g_value_take_string (offset_val, g_strdup_printf ("%d", MIDGARD_QUERY_EXECUTOR (self)->priv->offset));
+		g_value_take_string (offset_val, g_strdup_printf ("%d", executor->priv->offset));
 		offset_expr->value = offset_val;
 		sss->limit_offset = offset_expr;
-	}
+	}	
 
 	/* Check structure */
-	if (!gda_sql_statement_check_structure (sql_stm, &error)) {
-		g_warning (_("Can't build SELECT statement: %s)"),
-				error && error->message ? error->message : _("Unknown reason"));
-		if (error)
-			g_error_free (error);
+	if (!gda_sql_statement_check_structure (sql_stm, &err)) {
+		g_set_error (error, MIDGARD_EXECUTION_ERROR, MIDGARD_EXECUTION_ERROR_INTERNAL,
+				"Can't build SELECT statement: %s)", err && err->message ? err->message : _("Unknown reason"));
+		if (err)
+			g_clear_error (&err);
 		goto return_false;
 	} 
 
@@ -489,27 +718,30 @@ _midgard_query_select_execute (MidgardQueryExecutor *self)
 	}
 
 	/* execute statement */
-	GdaDataModel *model = gda_connection_statement_execute_select (cnc, stmt, NULL, &error);
+	GdaDataModel *model = gda_connection_statement_execute_select (cnc, stmt, NULL, &err);
 	g_object_unref (stmt);
 
-	if (!model && !error)
+	if (!model && !err) {
+		g_set_error (error, MIDGARD_EXECUTION_ERROR, MIDGARD_EXECUTION_ERROR_INTERNAL,
+				"Execute error - Unknown reason, underlying error is NULL");
 		goto return_false;
+	}
 
-	if (error) {
-		/* FIXME, set domain error code */
-		g_warning ("Execute error - %s", error->message);
-		g_error_free (error);
+	if (err) {
+		g_set_error (error, MIDGARD_EXECUTION_ERROR, MIDGARD_EXECUTION_ERROR_INTERNAL,
+				"Execute error - %s", err->message);
+		g_error_free (err);
 		goto return_false;
 	}
 	
-	MIDGARD_QUERY_EXECUTOR (self)->priv->results_count = gda_data_model_get_n_rows (model);
-	if (MIDGARD_QUERY_EXECUTOR (self)->priv->resultset && G_IS_OBJECT (MIDGARD_QUERY_EXECUTOR (self)->priv->resultset))
-		g_object_unref (G_OBJECT (MIDGARD_QUERY_EXECUTOR (self)->priv->resultset));
-	MIDGARD_QUERY_EXECUTOR (self)->priv->resultset = (gpointer) model;
+	executor->priv->results_count = gda_data_model_get_n_rows (model);
+	if (executor->priv->resultset && G_IS_OBJECT (executor->priv->resultset))
+		g_object_unref (G_OBJECT (executor->priv->resultset));
+	executor->priv->resultset = (gpointer) model;
 	g_object_unref (self);
 	
 	g_signal_emit (self, MIDGARD_QUERY_EXECUTOR_GET_CLASS (self)->signal_id_execution_end, 0);
-	return TRUE;
+	return;
 
 return_false:
 	if (sql_stm)
@@ -517,7 +749,7 @@ return_false:
 
 	g_signal_emit (self, MIDGARD_QUERY_EXECUTOR_GET_CLASS (self)->signal_id_execution_end, 0);
 	g_object_unref (self);
-	return FALSE;
+	return;
 }
 
 guint
@@ -568,7 +800,7 @@ _midgard_query_select_list_objects (MidgardQuerySelect *self, guint *n_objects)
 			}
 		} else {
 			MIDGARD_DBOBJECT_GET_CLASS (objects[i])->dbpriv->set_from_data_model (
-					MIDGARD_DBOBJECT (objects[i]), model, i);
+					MIDGARD_DBOBJECT (objects[i]), model, i, 0);
 		}
 	}
 
@@ -615,7 +847,7 @@ midgard_query_select_include_deleted (MidgardQuerySelect *self, gboolean toggle)
  *
  * List all objects for which data has been returned during execution.
  *
- * Returns: (element-type MidgardDBObject) (array length=n_objects): newly allocated array of #MidgardDBObject
+ * Returns: (element-type MidgardDBObject) (array length=n_objects) (transfer full): newly allocated array of #MidgardDBObject
  * Since: 10.05
  */ 
 MidgardDBObject **
@@ -641,6 +873,8 @@ _midgard_query_select_toggle_read_only (MidgardQuerySelect *self, gboolean toggl
  * It should be enabled when returned objects will be used only to read properties.
  * It improves performance, but it's impossible to write returned object's properties.
  *
+ * By default, for performance reason, MidgardQuerySelect is instantiated in read only mode.
+ *
  * Since: 10.05
  */ 
 void
@@ -691,6 +925,7 @@ _midgard_query_select_finalize (GObject *object)
 	GSList *l;
 	for (l = MIDGARD_QUERY_EXECUTOR (self)->priv->orders; l != NULL; l = l->next) {
 		qso *_so = (qso*) l->data;
+		g_free (_so->order_type);
 		g_object_unref (_so->property);
 	}
 
@@ -710,7 +945,7 @@ __midgard_query_select_get_property (GObject *object, guint property_id,
 			break;
 
 		case PROPERTY_STORAGE:
-			/* write and constructor only */
+			g_value_set_object (value, g_object_ref (MIDGARD_QUERY_EXECUTOR (self)->priv->storage));	
 			break;
 
 		default:
@@ -763,9 +998,8 @@ _midgard_query_select_class_init (MidgardQuerySelectClass *klass, gpointer class
 	executor_class->set_offset = _midgard_query_select_set_offset;
 	executor_class->add_order = _midgard_query_select_add_order;
 	executor_class->add_join = _midgard_query_select_add_join;
-	executor_class->execute = _midgard_query_select_execute;
 	executor_class->get_results_count = _midgard_query_select_get_results_count;
-	
+
 	klass->list_objects = _midgard_query_select_list_objects;
 	klass->toggle_read_only = _midgard_query_select_toggle_read_only;
 	klass->include_deleted = _midgard_query_select_include_deleted;
@@ -789,7 +1023,7 @@ _midgard_query_select_class_init (MidgardQuerySelectClass *klass, gpointer class
 	pspec = g_param_spec_object ("storage",
 			"", "", 
 			MIDGARD_TYPE_QUERY_STORAGE,
-			G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+			G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 	/**
 	 * MidgardQuerySelect:storage:
 	 * 
@@ -799,6 +1033,21 @@ _midgard_query_select_class_init (MidgardQuerySelectClass *klass, gpointer class
 	g_object_class_install_property (object_class, PROPERTY_STORAGE, pspec);
 }
 
+/* Executable iface */
+static void
+_midgard_query_select_executable_iface_init (MidgardExecutableIFace *iface)
+{
+	iface->execute = _midgard_query_select_executable_iface_execute;
+}
+
+/* Validable iface */
+static void
+_midgard_query_select_validable_iface_init (MidgardValidableIFace *iface)
+{
+	iface->is_valid = _midgard_query_select_validable_iface_is_valid;
+	iface->validate = _midgard_query_select_validable_iface_validate;
+}
+
 GType
 midgard_query_select_get_type (void)
 {
@@ -815,7 +1064,22 @@ midgard_query_select_get_type (void)
 			0,              /* n_preallocs */
 			NULL /* instance_init */
 		};
+
+		static const GInterfaceInfo executable_info = {
+			(GInterfaceInitFunc) _midgard_query_select_executable_iface_init,
+			NULL,   /* interface_finalize */
+			NULL    /* interface_data */
+		};
+
+		static const GInterfaceInfo validable_info = {
+			(GInterfaceInitFunc) _midgard_query_select_validable_iface_init,
+			NULL,   /* interface_finalize */
+			NULL    /* interface_data */
+		};
+
 		type = g_type_register_static (MIDGARD_TYPE_QUERY_EXECUTOR, "MidgardQuerySelect", &info, 0);
+		g_type_add_interface_static (type, MIDGARD_TYPE_EXECUTABLE, &executable_info);
+		g_type_add_interface_static (type, MIDGARD_TYPE_VALIDABLE, &validable_info);
 	}
 	return type;
 }
