@@ -24,12 +24,19 @@
 #include "midgard_core_query.h"
 #include "midgard_user.h"
 #include "midgard_core_connection.h"
+#include "midgard_core_metadata.h"
+#include "midgard_replicator.h"
+#include "midgard_core_workspace.h"
 
 #define ADMIN_USER_GUID "f4edf43cc32a11deb15c11e64583ae10ae10"
-#define ADMIN_PERSON_GUID "f6b665f1984503790ed91f39b11b5392"
 #define ADMIN_USER_LOGIN "admin"
 #define ADMIN_USER_PASSWORD "password"
 
+#define ADMIN_PERSON_GUID "f6b665f1984503790ed91f39b11b5392"
+#define ADMIN_PERSON_CREATED "1999-05-18 14:40:01"
+#define ADMIN_PERSON_FIRSTNAME "Midgard"
+#define ADMIN_PERSON_LASTNAME "Administrator"
+
 #define DEFAULT_AUTH_TYPE "Plaintext"
 
 struct _MidgardStorage {
@@ -102,28 +109,45 @@ midgard_storage_create_base_storage(MidgardConnection *mgd)
 		ret_rows = gda_data_model_get_n_rows(model);
 
 	if(!model || ret_rows == 0) {
-			
-		sql = "INSERT INTO person (guid, metadata_creator, lastname, firstname, metadata_created, metadata_revision) "
-			"VALUES ('f6b665f1984503790ed91f39b11b5392', 'f6b665f1984503790ed91f39b11b5392', "
-			"'Administrator', 'Midgard', '1999-05-18 14:40:01', 0 )";
-		midgard_core_query_execute(mgd, sql, TRUE);
+		
+		MidgardObject *person = midgard_object_new (mgd, "midgard_person", NULL);
+		/* guid */
+		g_free ((gchar *)MGD_OBJECT_GUID (person));
+		MGD_OBJECT_GUID (person) = g_strdup (ADMIN_PERSON_GUID);
+		/* lastname and firstname */
+		g_object_set (G_OBJECT (person), 
+			"lastname", ADMIN_PERSON_LASTNAME,
+			"firstname", ADMIN_PERSON_FIRSTNAME, NULL);
+		/* metadata creator */
+		GValue val = {0, };
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_string (&val, ADMIN_PERSON_GUID);
+		midgard_core_metadata_set_creator (MGD_DBOBJECT_METADATA (person), &val);
+		g_value_unset (&val);
+		//midgard_object_create (person);
 
-		/* Create root's repligard entry */
-		sql = "INSERT INTO repligard (guid, typename, object_action) "
-			"VALUES ('f6b665f1984503790ed91f39b11b5392', 'midgard_person', 0)";
-		midgard_core_query_execute(mgd, sql, TRUE);		
+		/* metadata created */
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_string (&val, ADMIN_PERSON_CREATED);
+		midgard_core_metadata_set_created (MGD_DBOBJECT_METADATA (person), &val);
+		g_value_unset (&val);
+
+		midgard_replicator_import_object (MIDGARD_DBOBJECT (person), TRUE);
 	}
 
 	if(model)
 		g_object_unref(model);
 
+	/* workspace table */
+	midgard_storage_create (mgd, "MidgardWorkspace");
+
 	return TRUE;
 }
 
 /**
  * midgard_storage_create:
  * @mgd: #MidgardConnection instance
- * @name: name of #MidgardDBObjectClass derived class
+ * @name: name of #MidgardDBObject derived class
  * 
  * Creates underlying storage (e.g. table in database) for class which is identified by given @name.
  * It may be class which represents any underlying storage type (database table or view, for example).
@@ -163,7 +187,10 @@ midgard_storage_create (MidgardConnection *mgd, const gchar *name)
 	g_return_val_if_fail (name != NULL, FALSE);
 
 	MidgardDBObjectClass *klass = g_type_class_peek (g_type_from_name (name));
-	g_return_val_if_fail(MIDGARD_IS_DBOBJECT_CLASS (klass), FALSE);
+	if (!MIDGARD_IS_DBOBJECT_CLASS (klass)) {
+		g_warning ("Expected DBObject derived instance. '%s' given", name);
+		return FALSE;
+	}
 
 	if (klass->dbpriv->create_storage == NULL)
 		return FALSE;
@@ -194,7 +221,10 @@ midgard_storage_update (MidgardConnection *mgd, const gchar *name)
 	g_return_val_if_fail (name != NULL, FALSE);
 
 	MidgardDBObjectClass *klass = g_type_class_peek (g_type_from_name (name));
-	g_return_val_if_fail(MIDGARD_IS_DBOBJECT_CLASS (klass), FALSE);
+	if (!MIDGARD_IS_DBOBJECT_CLASS (klass)) {
+		g_warning ("Expected DBObject derived instance. '%s' given", name);
+		return FALSE;
+	}
 
 	if (klass->dbpriv->update_storage == NULL)
 		return FALSE;
@@ -220,7 +250,10 @@ midgard_storage_exists (MidgardConnection *mgd, const gchar *name)
 	g_return_val_if_fail (name != NULL, FALSE);
 
 	MidgardDBObjectClass *klass = g_type_class_peek (g_type_from_name (name));
-	g_return_val_if_fail(MIDGARD_IS_DBOBJECT_CLASS (klass), FALSE);
+	if (!MIDGARD_IS_DBOBJECT_CLASS (klass)) {
+		g_warning ("Expected DBObject derived instance. '%s' given", name);
+		return FALSE;
+	}
 
 	if (klass->dbpriv->storage_exists == NULL)
 		return FALSE;
@@ -245,7 +278,10 @@ gboolean midgard_storage_delete (MidgardConnection *mgd, const gchar *name)
 	g_return_val_if_fail (name != NULL, FALSE);
 
 	MidgardDBObjectClass *klass = g_type_class_peek (g_type_from_name (name));
-	g_return_val_if_fail(MIDGARD_IS_DBOBJECT_CLASS (klass), FALSE);
+	if (!MIDGARD_IS_DBOBJECT_CLASS (klass)) {
+		g_warning ("Expected DBObject derived instance. '%s' given", name);
+		return FALSE;
+	}
 
 	if (klass->dbpriv->delete_storage == NULL)
 		return FALSE;
